#!/usr/bin/python

#IDEA: option for sorting threads differently: interleave sorted by started with sorted by updated

import sys, os, logging, logging.handlers, socket

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler())
fh = logging.handlers.RotatingFileHandler(os.path.join(os.path.dirname(__file__), 'log'), maxBytes = 10*1024*1024, backupCount = 3)
fh.setFormatter(logging.Formatter('%(asctime)s %(name)s %(levelname)s %(message)s'))
logger.addHandler(fh)

try:
	import config				#config.py is kept in this folder not img2/ for ease of access
	os.unlink('config.pyc')
except ImportError:
	logging.critical('cant find config.py, you need to rename "config.py.example" to "config.py" and edit the settings in it')
	sys.exit()

if not config.UNIQUE_STRING:
	logging.critical('you need to set the UNIQUE_STRING in config.py')
	sys.exit()

if not config.BOARDS:
	logging.critical('you need to define at least one board in config.py')
	sys.exit()

import img2.database
try:
	img2.database.check()
except:
	logging.exception('error checking database')
	sys.exit()

import img2.cgi2, img2.cgi2.scgi, img2.cgi2.fastcgi, img2.cgi2.sessions, img2.views
try:
	server = {'scgi': img2.cgi2.scgi, 'fastcgi': img2.cgi2.fastcgi}[config.SOCKET[0]]
	socket_addr = config.SOCKET[1:]
	request_handler = img2.views.handler
	session_handler = img2.cgi2.sessions.SQLiteHandler('databases/sessions.sqlite')
	
	server.listen(socket_addr, request_handler, session_handler)
except:
	logging.exception('unable to start server')
