from __future__ import with_statement

import re, cgi, urllib, threading, time, socket, Queue, signal, errno, logging, os
import sessions


class Request:
	def __init__(self, headers, file, session_handler):
		self.headers = headers
		self.session_handler = session_handler
		
		#make sure all required variables exist
		if 'CONTENT_LENGTH' not in headers:
			headers['CONTENT_LENGTH'] = '0'
		if 'CONTENT_TYPE' not in headers:
			headers['CONTENT_TYPE'] = 'application/octet-stream'
		
		#parse GET
		self.get_vars = cgi.parse_qs(headers['QUERY_STRING'], keep_blank_values = True)
		for k in self.get_vars:
			d = self.get_vars[k]
			for i, v in enumerate(d):
				d[i] = unicode(v, 'utf-8', 'replace')
		
		#parse POST
		self.post_vars = {}
		self.files = {}
		ct = cgi.parse_header(self['CONTENT_TYPE'])
		
		if ct[0] == 'application/x-www-form-urlencoded':
			self.post_vars = cgi.parse_qs(file.read(int(headers['CONTENT_LENGTH'])), keep_blank_values = True)
			for k in self.post_vars:
				d = self.post_vars[k]
				for i, v in enumerate(d):
					d[i] = unicode(v, 'utf-8', 'replace')
					
		elif ct[0] == 'multipart/form-data':
			fs = cgi.FieldStorage(fp = file, environ = headers, keep_blank_values = True)
			for k in fs:
				val = fs[k]
				if not isinstance(val, list) and val.name is not None:
					if val.filename:
						self.files[unicode(val.name, 'utf-8', 'replace')] = UploadedFile(unicode(val.filename, 'utf-8', 'replace'), val.file)
					else:
						self.post_vars[unicode(val.name, 'utf-8', 'replace')] = [unicode(val.value, 'utf-8', 'replace')]
		
		#parse cookies
		self.cookies = {}
		cx = self['HTTP_COOKIE'].split(';')
		for c in cx:
			nv = c.strip().split('=')
			if len(nv) == 2:
				nv[0] = unicode(urllib.unquote_plus(nv[0]), 'utf-8', 'replace')
				nv[1] = unicode(urllib.unquote_plus(nv[1]), 'utf-8', 'replace')
				self.cookies[nv[0]] = nv[1]
		
		self.session_id = sessions.get_valid_id(self.COOKIE('sid'))
		self.session = session_handler.load(self.session_id)
		self.time = time.time()
	
	def __getitem__(self, key):
		return self.headers.get(key, '')
	
	def __getattr__(self, name):
		if name == 'method':
			return self.headers['REQUEST_METHOD']
		raise AttributeError
	
	def GET(self, key, default = None, index = 0):
		if key in self.get_vars and index < len(self.get_vars[key]):
			return self.get_vars[key][index]
		else:
			return default
	
	def POST(self, key, default = None, index = 0):
		if key in self.post_vars and index < len(self.post_vars[key]):
			return self.post_vars[key][index]
		else:
			return default

	def FILE(self, key, default = None):
		return self.files.get(key, default)

	def COOKIE(self, key, default = None):
		return self.cookies.get(key, default)

	def is_ajax(self):
		return self['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest'
	
	def local_referer(self):
		sn = self['SERVER_NAME']
		rf = self['HTTP_REFERER'].partition('://')[2]
		return rf.startswith(sn + '/') or (rf == sn)

	def accepts_gzip(self):
		ae = self['HTTP_ACCEPT_ENCODING']
		if not ae:
			return False
		else:
			eq = {}
			for e in [x.strip() for x in ae.split(',')]:
				x = e.split(';')
				if len(x) == 1:
					eq[x[0]] = 1
				else:
					try:
						eq[x[0]] = float(x[1].partition('=')[2])
					except:
						eq[x[0]] = 0
			return (eq.get('gzip', 0) > 0) or (eq.get('*', 0) > 0 and eq.get('gzip') != 0)

	def store_session(self):
		self.session_handler.store(self.session_id, self.session)
	
	def set_sid_cookie(self, response):
		if self.COOKIE('sid') != self.session_id:
			response.set_cookie('sid', self.session_id, 60*60*24, path = '/')


class Response:
	def __init__(self, body = '', type = 'text/html; charset=utf-8', status = '200 OK'):
		self.headers = {'Status': status}
		self.set_body(body, type)
		self.cookies = {}	#name => Cookie
	
	def __setitem__(self, key, value):
		self.headers[key] = value
	
	def __getitem__(self, key):
		return self.headers[key]
	
	def set_body(self, body, type = 'text/html; charset=utf-8'):
		if isinstance(body, unicode):
			body = body.encode('utf-8', 'replace')
		self.body = body
		self['Content-Type'] = type
		self['Content-Length'] = len(body)
	
	def set_cookie(self, name, value, max_age = None, expires = None, path = None, domain = None, secure = False, http_only = False, raw = False):
		if expires is None and max_age is not None:
			expires = time.time() + max_age
		self.cookies[name] = Cookie(name, value, expires, path, domain, secure, http_only, raw)
	
	def delete_cookie(self, name, path = None, domain = None, secure = False, http_only = False):
		self.set_cookie(name, '', -24*60*60, None, path, domain, secure, http_only)
	
	def write(self, file, request):
		for k, v in self.headers.items():
			file.write(k)
			file.write(': ')
			file.write(str(v))
			file.write('\r\n')
		self.write_cookies(file)
		file.write('\r\n')
		file.write(self.body)

	def write_cookies(self, file):
		for name, cookie in self.cookies.items():
			file.write('Set-Cookie: ')
			file.write(urllib.quote(name.encode('utf-8', 'replace')))
			file.write('=')
			if not isinstance(cookie.value, basestring):
				cookie.value = str(cookie.value)
			if cookie.raw:
				file.write(cookie.value.encode('utf-8', 'replace'))
			else:
				file.write(urllib.quote(cookie.value.encode('utf-8', 'replace')))
			if cookie.expires is not None:
				file.write('; expires=')
				file.write(time.strftime('%a, %d-%b-%Y %H:%M:%S GMT', time.gmtime(cookie.expires)))
			if cookie.path is not None:
				file.write('; path=')
				file.write(cookie.path)
			if cookie.domain is not None:
				file.write('; domain=')
				file.write(cookie.domain)
			if cookie.secure:
				file.write('; secure')
			if cookie.http_only:
				file.write('; http_only')
			file.write('\r\n')
	

class RedirectResponse(Response):
	def __init__(self, path):
		Response.__init__(self)
		self.path = path
	
	def write(self, file, request):
		file.write('Location: ')
		p = self.path
		if p.startswith('/'):
			p = 'http://' + request['SERVER_NAME'] + p
		file.write(p)
		file.write('\r\n')
		self.write_cookies(file)
		file.write('\r\n')


class UploadedFile:
	def __init__(self, name, file):
		self.name = name
		self.file = file


class Cookie:
	def __init__(self, name, value, expires, path, domain, secure, http_only, raw):
		self.name = name
		self.value = value
		self.expires = expires
		self.path = path
		self.domain = domain
		self.secure = secure
		self.http_only = http_only
		self.raw = raw
	def __str__(self):
		return 'Cookie: %s' % self.__dict__


def escape(input):
	if isinstance(input, (int, long, float, bool)):
		return str(input)
	elif isinstance(input, basestring):
		return input.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;').replace('\'', '&#039;')
	else:
		raise TypeError('arg not int, long, float, boolean, str or unicode')


def escape_nl(input):
	return escape(input).replace('\r\n', '<br>')


def strip_html(input):
	return re.sub(r'<.*?>', '', input)


def nl_to_br(input):
	return input.replace('\r\n', '<br>')


def read_checked(file, rlen = 1):
	r = file.read(rlen)
	if len(r) < rlen:
		raise EOFError
	return r


class RequestThread(threading.Thread):
	""" grab conn from queue, feed it to handler, repeat """
	
	def __init__(self, server):
		threading.Thread.__init__(self)
		self.server = server

	def run(self):
		while True:
			x = self.server.request_queue.get()
			if x is None:
				break
			try:
				sock, addr = x
				file = sock.makefile()
				self.server.conn_handler.handle(file)
				file.close()
				sock.close()
			except Exception, e:
				#EPIPE means client closed connection before it was complete (at least with scgi) - ignorable
				if not (isinstance(e, socket.error) and e[0] == errno.EPIPE):
					logging.exception('request thread exception')
			self.server.request_queue.task_done()


class ThreadedServer:
	def __init__(self, addr, conn_handler, max_threads = 10):
		
		self.conn_handler = conn_handler
		self.request_queue = Queue.Queue()
		
		threads = [RequestThread(self)]
		threads[0].start()
		
		sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		sock.bind(addr)
		sock.listen(5)
		logging.info('%s server listening on %s, (ctrl-c to quit) ...' % (conn_handler.name, addr))
		
		quit = False
		def set_quit(number, frame):
			quit = True
		signal.signal(signal.SIGINT, set_quit)
		signal.signal(signal.SIGTERM, set_quit)
		
		while not quit:
			try:
				ns = sock.accept()
				if not self.request_queue.empty() and len(threads) < max_threads:
					t = RequestThread(self)
					t.start()
					threads.append(t)
					logging.info('now using %d handler threads' % len(threads))
				self.request_queue.put(ns)
			except socket.error, e:
				if e[0] == errno.EINTR:		#interupted socket op
					quit = True
					break
				else:
					logging.exception('server accept socket error')
					os._exit(1)
			except:
				logging.exception('server accept error')
				os._exit(1)
		
		sock.close()
		for t in threads:
			self.request_queue.put(None)
		for t in threads:
			t.join()

# try and get faster versions of functions
try:
	from ccgi2 import *
except ImportError:
	pass