#include <Python.h>


#define COPY_ENT(a, b, to) *(to++) = '&'; *(to++) = '#'; *(to++) = a; *(to++) = b; *(to++) = ';';
#define COPY_BR(to) *(to++) = '<'; *(to++) = 'b'; *(to++) = 'r'; *(to++) = '>';


static PyObject *ccgi2_escape(PyObject *self, PyObject *arg){
	
	if(PyInt_Check(arg) || PyLong_Check(arg) || PyFloat_Check(arg) || PyBool_Check(arg))
		return PyObject_Str(arg);
		
	if(PyUnicode_Check(arg)){
		
		Py_UNICODE *str, *ptr;
		Py_ssize_t len;
		int i, add_len;
		int start, end;
		
		str = PyUnicode_AsUnicode(arg);
		len = PyUnicode_GetSize(arg);
		
		add_len = 0;
		start = -1, end = len;
		ptr = str;
		for(i=0; i<len; i++){
			Py_UNICODE c = *(ptr++);
			if(c == '<' || c == '>' || c == '&' || c == '\'' || c == '"'){
				add_len++;
				if(start == -1)
					start = i;
				end = i+1;
			}
		}
		add_len *= 4;
		
		
		if(add_len){
			PyObject *new_str_obj = PyUnicode_FromUnicode(NULL, len + add_len);
			if(new_str_obj == NULL){
				PyErr_SetString(PyExc_MemoryError, "unable to allocate new unicode object");
				return NULL;
			}
			Py_UNICODE *new_str = PyUnicode_AS_UNICODE(new_str_obj);
			memcpy(new_str, str, start * sizeof(Py_UNICODE));
			Py_UNICODE *from_ptr = str+start, *ptr = new_str+start;			
			for(i=start; i<end; i++){
				Py_UNICODE c = *(from_ptr++);
				if(c == '<'){			COPY_ENT('6', '0', ptr)
				}else if(c == '>'){		COPY_ENT('6', '2', ptr)
				}else if(c == '&'){		COPY_ENT('3', '8', ptr)
				}else if(c == '"'){		COPY_ENT('3', '4', ptr)
				}else if(c == '\''){	COPY_ENT('3', '9', ptr)
				}else
					*(ptr++) = c;
			}
			memcpy(ptr, from_ptr, (len-end) * sizeof(Py_UNICODE));
			return new_str_obj;
		}
		else{
			Py_INCREF(arg);
			return arg;
		}
	}
	
	if(PyString_Check(arg)){
	
		char *str, *ptr;
		Py_ssize_t len;
		int i, add_len;
		int start, end;
		
		PyString_AsStringAndSize(arg, &str, &len);
	
		add_len = 0;
		start = -1, end = len;
		ptr = str;
		for(i=0; i<len; i++){
			char c = *(ptr++);
			if(c == '<' || c == '>' || c == '&' || c == '\'' || c == '"'){
				add_len++;
				if(start == -1)
					start = i;
				end = i+1;
			}
		}
		add_len *= 4;
		
		if(add_len){
			PyObject *new_str_obj = PyString_FromStringAndSize(NULL, len + add_len);
			if(new_str_obj == NULL){
				PyErr_SetString(PyExc_MemoryError, "unable to allocate new str object");
				return NULL;
			}
			char *new_str = PyString_AsString(new_str_obj);
			memcpy(new_str, str, start);
			char *from_ptr = str+start, *ptr = new_str+start;
			for(i=start; i<end; i++){
				char c = *(from_ptr++);
				if(c == '<'){			COPY_ENT('6', '0', ptr)
				}else if(c == '>'){		COPY_ENT('6', '2', ptr)
				}else if(c == '&'){		COPY_ENT('3', '8', ptr)
				}else if(c == '"'){		COPY_ENT('3', '4', ptr)
				}else if(c == '\''){	COPY_ENT('3', '9', ptr)
				}else
					*(ptr++) = c;
			}
			memcpy(ptr, from_ptr, len-end);
			return new_str_obj;
		}
		else{
			Py_INCREF(arg);
			return arg;
		}
	}
	
	PyErr_SetString(PyExc_TypeError, "arg not int, long, float, boolean, str or unicode");
	return NULL;
}


static PyObject *ccgi2_escape_nl(PyObject *self, PyObject *arg){
	
	if(PyInt_Check(arg) || PyLong_Check(arg) || PyFloat_Check(arg) || PyBool_Check(arg))
		return PyObject_Str(arg);
		
	if(PyUnicode_Check(arg)){
		
		Py_UNICODE *str, *ptr;
		Py_ssize_t len;
		int i, add_len;
		int start, end;
		char prev;
		
		str = PyUnicode_AsUnicode(arg);
		len = PyUnicode_GetSize(arg);
		
		add_len = 0;
		start = -1, end = len;
		ptr = str;
		prev = 0;
		for(i=0; i<len; i++){
			Py_UNICODE c = *(ptr++);
			if(c == '<' || c == '>' || c == '&' || c == '\'' || c == '"'){
				add_len += 4;
				if(start == -1)
					start = i;
				end = i+1;
			}
			else if(c == '\n' && prev == '\r'){
				add_len += 2;
				if(start == -1)
					start = i-1;
				end = i+1;
			}
			prev = c;
		}
		
		
		if(add_len){
			PyObject *new_str_obj = PyUnicode_FromUnicode(NULL, len + add_len);
			if(new_str_obj == NULL){
				PyErr_SetString(PyExc_MemoryError, "unable to allocate new unicode object");
				return NULL;
			}
			Py_UNICODE *new_str = PyUnicode_AS_UNICODE(new_str_obj);
			memcpy(new_str, str, start * sizeof(Py_UNICODE));
			Py_UNICODE *from_ptr = str+start, *ptr = new_str+start;			
			for(i=start; i<end; i++){
				Py_UNICODE c = *(from_ptr++);
				if(c == '<'){			COPY_ENT('6', '0', ptr)
				}else if(c == '>'){		COPY_ENT('6', '2', ptr)
				}else if(c == '&'){		COPY_ENT('3', '8', ptr)
				}else if(c == '"'){		COPY_ENT('3', '4', ptr)
				}else if(c == '\''){	COPY_ENT('3', '9', ptr)
				}else if(c == '\r' && i < end-1 && *from_ptr == '\n'){
					COPY_BR(ptr)
					from_ptr++;
					i++;
				}else
					*(ptr++) = c;
			}
			memcpy(ptr, from_ptr, (len-end) * sizeof(Py_UNICODE));
			return new_str_obj;
		}
		else{
			Py_INCREF(arg);
			return arg;
		}
	}
	
	if(PyString_Check(arg)){
	
		char *str, *ptr;
		Py_ssize_t len;
		int i, add_len;
		int start, end;
		char prev;
		
		PyString_AsStringAndSize(arg, &str, &len);
	
		add_len = 0;
		start = -1, end = len;
		ptr = str;
		prev = 0;
		for(i=0; i<len; i++){
			char c = *(ptr++);
			if(c == '<' || c == '>' || c == '&' || c == '\'' || c == '"'){
				add_len += 4;
				if(start == -1)
					start = i;
				end = i+1;
			}			
			else if(c == '\n' && prev == '\r'){
				add_len += 2;
				if(start == -1)
					start = i-1;
				end = i+1;
			}
			prev = c;
		}
		
		if(add_len){
			PyObject *new_str_obj = PyString_FromStringAndSize(NULL, len + add_len);
			if(new_str_obj == NULL){
				PyErr_SetString(PyExc_MemoryError, "unable to allocate new str object");
				return NULL;
			}
			char *new_str = PyString_AsString(new_str_obj);
			memcpy(new_str, str, start);
			char *from_ptr = str+start, *ptr = new_str+start;
			for(i=start; i<end; i++){
				char c = *(from_ptr++);
				if(c == '<'){			COPY_ENT('6', '0', ptr)
				}else if(c == '>'){		COPY_ENT('6', '2', ptr)
				}else if(c == '&'){		COPY_ENT('3', '8', ptr)
				}else if(c == '"'){		COPY_ENT('3', '4', ptr)
				}else if(c == '\''){	COPY_ENT('3', '9', ptr)
				}else if(c == '\r' && i < end-1 && *from_ptr == '\n'){
					COPY_BR(ptr)
					from_ptr++;
					i++;
				}else
					*(ptr++) = c;
			}
			memcpy(ptr, from_ptr, len-end);
			return new_str_obj;
		}
		else{
			Py_INCREF(arg);
			return arg;
		}
	}
	
	PyErr_SetString(PyExc_TypeError, "arg not int, long, float, boolean, str or unicode");
	return NULL;
}


static PyMethodDef ccgi2_methods[] = {
	{"escape", ccgi2_escape, METH_O, "if arg is a int, long, float or boolean return str(arg), if arg is a str or unicode return a html escaped version, otherwise raise TypeError"},
	{"escape_nl", ccgi2_escape_nl, METH_O, "like escape but also replaces \\r\\n by <br>"},
	{NULL, NULL}
};



PyMODINIT_FUNC initccgi2(void){
	Py_InitModule3("ccgi2", ccgi2_methods, "cgi/web helper module");
}
