""" SCGI connection handler """

from __future__ import with_statement

from __init__ import ThreadedServer, Request, Response, read_checked


class SCGIException(Exception):
	""" thrown when the SCGI protocol has been broken """


def read_netstring(file):
	n = ''
	while True:
		x = read_checked(file)
		if x == ':':
				load = read_checked(file, int(n))
				read_checked(file) # skip ','
				return load
		else:
			n += x;


class ConnectionHandler:
	def __init__(self, request_handler, session_handler):
		self.request_handler = request_handler
		self.session_handler = session_handler
		self.name = 'scgi'
	
	def handle(self, file):
		hx = read_netstring(file).split('\0')
		headers = dict(zip(hx[::2], hx[1::2]))
		request = Request(headers, file, self.session_handler)
		response = self.request_handler(request)
		request.store_session()
		request.set_sid_cookie(response)
		response.write(file, request)


def listen(addr, request_handler, session_handler):
	ThreadedServer(addr, ConnectionHandler(request_handler, session_handler))