from __future__ import with_statement

import os, random, time, threading
import config
from util import ObjDict	#used in config.py which imports * from here


SECONDS_REQUESTS_CONSIDERED_ACTIVE = 60*4


class Board:
	""" config.py defines an instance of this for each board """
	
	def __init__(self, **d):
		self.__dict__.update(config.BOARD_DEFAULTS)
		self.__dict__.update(d)
		self.recent_requests = []		# list of [sid, request_count, time_of_last_request]
		self.recent_requests_lock = threading.Lock()
	
	
	def request(self, request):
		""" called when a request arrives for this board """
		with self.recent_requests_lock:
			now = time.time()
			for s in self.recent_requests:
				if s[0] == request.session_id:
					s[1] += 1
					s[2] = now
					break
			else:
				self.recent_requests.append([request.session_id, 1, now])
			# remove out-of-date entries
			self.recent_requests = [e for e in self.recent_requests if e[2] > now - SECONDS_REQUESTS_CONSIDERED_ACTIVE]
	
	def active_visitors(self):
		with self.recent_requests_lock:
			n = 0
			for s in self.recent_requests:
				if s[1] > 1:
					n += 1
			return n
	
	def get_default_name(self):
		f = self.DEFAULT_FIRST_NAMES
		s = self.DEFAULT_SECOND_NAMES
		if f and s:
			return random.choice(f) + ' ' + random.choice(s)
		elif f:
			return random.choice(f)
		elif s:
			random.choice(s)
		else:
			return ''
	
	def db(self):
		import database
		return database.BoardDataBase(self)
	
	def get_file_dir(self):
		import config
		return os.path.join(config.UPLOADED_FILE_DIR, self.DIR)
	
	def __cmp__(self, other):
		return cmp(self.DIR, other)
	
	def __hash__(self):
		return hash(self.DIR)