import os, sqlite3, math, time, cPickle, random, re, socket, struct
import config, common, uploaded_files, util, parser, tripcodes, sessions

POSTS_STATUS = ['undeleted', 'user_deleted', 'mod_deleted']

GENERAL_TABLES = """

CREATE TABLE IF NOT EXISTS bans (
	ip TEXT PRIMARY KEY NOT NULL,
	expires INTEGER NOT NULL,
	reason TEXT NOT NULL
);

CREATE TABLE IF NOT EXISTS file_bans (
	md5 BLOB PRIMARY KEY NOT NULL
);

"""

BOARD_TABLES = """
	
CREATE TABLE IF NOT EXISTS threads (
	id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
	forked_thread_id INTEGER,
	forked_post_id INTEGER,
	subject TEXT NOT NULL,
	sticky INTEGER NOT NULL,
	locked INTEGER NOT NULL,
	updated INTEGER NOT NULL
);
CREATE INDEX IF NOT EXISTS threads_updated ON threads (updated);
CREATE INDEX IF NOT EXISTS threads_forked ON threads (forked_thread_id, forked_post_id);

CREATE TABLE IF NOT EXISTS posts (
	thread_id INTEGER NOT NULL,
	id INTEGER NOT NULL,
	posted INTEGER NOT NULL,
	name TEXT NOT NULL,
	tripcode TEXT,
	mod INTEGER NOT NULL,
	ip TEXT,
	comment TEXT NOT NULL,
	file_md5 BLOB,
	file_meta BLOB,
	status INTEGER NOT NULL,
	PRIMARY KEY (thread_id, id)
);
CREATE INDEX IF NOT EXISTS posts_posted ON posts (posted);
CREATE INDEX IF NOT EXISTS posts_file_md5 ON posts (file_md5);

CREATE TABLE IF NOT EXISTS flags (
	thread_id INTEGER NOT NULL,
	post_id INTEGER NOT NULL,
	ip TEXT NOT NULL,
	PRIMARY KEY (thread_id, post_id, ip)
);

CREATE TABLE IF NOT EXISTS reviewed_posts (
	thread_id INTEGER NOT NULL,
	post_id INTEGER NOT NULL,
	time INTEGER NOT NULL,
	PRIMARY KEY (thread_id, post_id)
);

"""


def check():
	try:
		os.makedirs(os.path.join(config.BASE_PATH, 'databases/boards'))
	except:
		pass
	GeneralDataBase().check()
	for board in config.BOARDS:
		BoardDataBase(board).check()


def cleanup():
	GeneralDataBase().delete_expired_bans()
	for board in config.BOARDS:
		board.db().anonymize_old_posts()


def dict_factory(cursor, row):
	d = {}
	for i, col in enumerate(cursor.description):
		d[col[0]] = row[i]
	return d


def inow():
	return int(time.time())


def get_mod_area_overview_data():
	boards = {}
	for board in config.BOARDS:
		boards[board] = board.db().get_mod_area_overview_data()
	return {'boards': boards}


def move_thread(src_board, src_thread_id, dest_board, server_host):
	src_db = src_board.db()
	dest_db = dest_board.db()
	
	# grab data from src database
	src_db.begin()
	thread = src_db.select_1("SELECT * FROM threads WHERE id = ?", src_thread_id)
	posts = src_db.select("SELECT * FROM posts WHERE thread_id = ?", src_thread_id)
	src_db.commit()
	
	if thread:
		# put data into dest database
		dest_db.begin(immediate = True)
		
		q = "INSERT INTO threads (subject, sticky, locked, updated, forked_thread_id, forked_post_id) VALUES (?, ?, ?, inow(), NULL, NULL)"
		dest_db.insert(q, (thread['subject'], thread['sticky'], thread['locked']))
		dest_thread_id = dest_db.cur.lastrowid

		for post in posts:
			q = """	INSERT INTO posts (thread_id, id, posted, status, name, tripcode, mod, comment, file_md5, file_meta, ip)
					VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"""
			dest_db.insert(q, (dest_thread_id, post['id'], post['posted'], post['status'], post['name'], post['tripcode'], post['mod'], post['comment'], post['file_md5'], post['file_meta'], post['ip']))
			if post['file_meta']:
				f = cPickle.loads(str(post['file_meta']))
				f.copy(common.Post(src_board, src_thread_id, post['id']), common.Post(dest_board, dest_thread_id, post['id']))
		dest_db.commit()
		
		# link src to dest thread and lock src
		comment = "Please continue this thread in http://%s/%s/%d" % (server_host, dest_board.DIR, dest_thread_id)
		src_db.add_new_reply(src_thread_id, '', None, True, comment, None, None, None, False, True)
		return dest_thread_id
	else:
		return None


class DataBase:
	""" superclass with setup/utility methods """
	
	
	def __init__(self, path):
		self.query_count = 0
		self.db = sqlite3.connect(path)
		self.db.isolation_level = None
		self.db.row_factory = dict_factory
		self.db.create_function('inow', 0, inow)
		self.db.create_function('format_time', 0, lambda: util.format_time(None, config.GMT_OFFSET))
		self.db.create_function('format_time', 1, lambda t: util.format_time(t, config.GMT_OFFSET))
		self.db.create_function('format_status', 1, lambda s: POSTS_STATUS[s])
		self.db.create_function('unhex', 1, lambda x: buffer(util.unhex(x)))
		self.db.create_function('fqdn', 1, lambda x: unicode(socket.getfqdn(x)) if x else None)
		self.cur = self.db.cursor()


	def insert(self, query, params = ()):
		"""does the query, returns nothing"""
		if params == ():
			self.cur.execute(query)
		else:
			self.cur.execute(query, util.tuplize(params))
		self.query_count += 1


	def select(self, query, params = ()):
		"""does the query and returns the fetchall() result in one call"""
		if params == ():
			self.cur.execute(query)
		else:
			self.cur.execute(query, util.tuplize(params))
		self.query_count += 1
		return self.cur.fetchall()


	def select_1(self, query, params = ()):
		"""does the query and returns the first returned row, or None in one call"""
		data = self.select(query, params)
		return data[0] if data else None


	def begin(self, immediate = False):
		self.cur.execute("BEGIN IMMEDIATE" if immediate else "BEGIN")
	
	
	def vacuum(self):
		self.insert("VACUUM")


	def commit(self):
		self.cur.execute("COMMIT")
	
	
	def rollback(self):
		self.cur.execute("ROLLBACK")


class GeneralDataBase(DataBase):
	""" the global/general database contains ip/file bans """
	

	def __init__(self):
		DataBase.__init__(self, 'databases/general.sqlite')


	def check(self):
		self.cur.executescript(GENERAL_TABLES)
		self.vacuum()


	def get_ban(self, ip):
		q = """SELECT ip, fqdn(ip) as fqdn, format_time(expires) AS expires, reason FROM bans WHERE ip = ? AND expires > inow()"""
		return self.select_1(q, ip)
	

	def add_ban(self, ip, expires, reason):
		q = "INSERT OR REPLACE INTO bans (ip, expires, reason) VALUES (?, ?, ?)"
		self.insert(q, (ip, expires, reason))


	def delete_ban(self, ip):
		self.insert("DELETE FROM bans WHERE ip = ?", ip)


	def delete_expired_bans(self):
		self.insert("DELETE FROM bans WHERE expires <= inow()")
	
	
	def file_banned(self, md5):
		return self.select_1("SELECT 1 FROM file_bans WHERE md5 = ?", buffer(md5)) is not None
	

	def add_file_ban(self, md5):
		self.insert("INSERT OR IGNORE INTO file_bans (md5) VALUES (?)", buffer(md5))


	def ban_files(self, md5s):
		self.begin()
		q = "INSERT OR IGNORE INTO file_bans (md5) VALUES (unhex(?))"
		for md5 in md5s:
			self.insert(q, md5)
		self.commit()


	def unban_files(self, md5s):
		self.begin()
		q = "DELETE FROM file_bans WHERE md5 = unhex(?)"
		for md5 in md5s:
			self.insert(q, md5)
		self.commit()


	def delete_expired_bans(self):
		self.insert("DELETE FROM bans WHERE expires <= inow()")


	def get_mod_area_bans_data(self, filter):
		
		try:
			f = filter.split('/')
			ip = struct.unpack('!L', socket.inet_aton(f[0]))[0]
			if len(f) == 1:
				filter = lambda t_ip: t_ip == f[0]
			else:
				nl = util.clamp(f[1], 0, 32)
				hostmask = int('1'*nl + '0'*(32-nl), 2)
				def filter(t_ip):
					try:
						return (struct.unpack('!L', socket.inet_aton(t_ip))[0] & hostmask) == (ip & hostmask)
					except:
						return False
		except:
			filter = lambda t_ip: True
		
		self.db.create_function('filter', 1, filter)
		q = """	SELECT ip, expires - inow() as expires, reason
				FROM bans
				WHERE expires > inow() AND filter(ip)
				ORDER BY expires"""
		bans = self.select(q)
		for ban in bans:
			ban['expires'] = common.format_time(ban['expires'], 2)
		return {'bans': bans}
	

	def get_mod_area_files_data(self):
		return self.select_1("SELECT COUNT(*) AS num_file_bans FROM file_bans")



class BoardDataBase(DataBase):
	""" each board has its own database, containing threads/posts + flagged/reviewed posts """


	def __init__(self, board):
		DataBase.__init__(self, 'databases/boards/%s.sqlite' % board.DIR)
		self.board = board
	
	
	def check(self):
		self.cur.executescript(BOARD_TABLES)
		self.vacuum()
	
	
	def get_forked_post_data(self, thread_id, post_id, display_thread_id = -1):
		#display_thread_id will be excluded from the list of other forked threads
		q = """	SELECT thread_id, id, name, format_time(posted) AS posted, tripcode, mod, format_status(status) AS status, comment, file_meta,
				EXISTS (SELECT 1 FROM threads WHERE forked_thread_id = posts.thread_id and forked_post_id = posts.id AND id != ?) AS has_forked_threads
				FROM posts
				WHERE thread_id = ? AND posts.id = ?"""
		post = self.select_1(q, (display_thread_id, thread_id, post_id))
		if post is None:
			return {'status': 'mod_deleted'}
		if post['status'] != 'undeleted':
			return {'status': post['status']}
		post['file_meta'] = cPickle.loads(str(post['file_meta'])) if post['file_meta'] else None
		post['comment'] = parser.parse(post['comment'], self.board, (thread_id, post_id, False))
		#get list of existing threads forked from this post
		if post['has_forked_threads'] > 0:
			q = """	SELECT id, subject FROM threads
					WHERE forked_thread_id = ? AND forked_post_id = ? AND thread_id != ?
					ORDER BY id"""
			post['forked_threads'] = self.select(q, (thread_id, post_id, display_thread_id))
		return post


	def get_view_board_data(self, page):
		self.begin()
		q = "SELECT COUNT(*) AS count FROM threads"
		num_threads = self.select_1(q)['count']
		num_pages = int(math.ceil(num_threads / float(self.board.THREADS_PER_PAGE)))
		pages = range(max(1, num_pages))
		#get list of threads
		q = """	SELECT *,
				(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id) AS num_posts,
				(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id AND file_md5 IS NOT NULL) AS num_files
				FROM threads
				ORDER BY sticky DESC, updated DESC LIMIT ?, ?"""
		threads = self.select(q, (self.board.THREADS_PER_PAGE * page, self.board.THREADS_PER_PAGE))
		#no threads on this page?
		if page > 0 and len(threads) == 0:
			self.commit()
			return None
		#get list of posts for each thread
		q = """	SELECT * FROM (
					SELECT thread_id, id, format_time(posted) AS posted, name, tripcode, mod, format_status(status) AS status, comment, file_meta,
					EXISTS (SELECT 1 FROM threads WHERE forked_thread_id = posts.thread_id AND forked_post_id = posts.id) AS has_forked_threads
					FROM posts
					WHERE thread_id = ?
					ORDER BY (id = 0) DESC, posts.id DESC LIMIT ?
				) ORDER BY id ASC"""
		q2 =  """	SELECT id, subject
					FROM threads
					WHERE forked_thread_id = ? AND forked_post_id = ?
					ORDER BY id"""
		for thread in threads:
			#get forked post data?
			if thread['forked_thread_id'] is not None and thread['forked_post_id'] is not None:
				thread['forked_post'] = self.get_forked_post_data(thread['forked_thread_id'], thread['forked_post_id'], thread['id'])
			else:
				thread['forked_post'] = None
			#get posts data
			if (thread['num_posts']-1) >= self.board.MIN_REPLIES_FOR_OMITTING:
				limit = self.board.NUM_REPLIES_TO_SHOW_WHEN_OMITTING + 1
			else:
				limit = -1
			posts = self.select(q, (thread['id'], limit))
			#parse posts for display
			files_shown = 0
			for post in posts:
				if post['file_meta']:
					post['file_meta'] = cPickle.loads(str(post['file_meta']))
					files_shown += 1
				post['comment'] = parser.parse(post['comment'], self.board, (thread['id'], post['id'], False))
				#get list of threads forked from this post
				if post['has_forked_threads']:
					post['forked_threads'] = self.select(q2, (thread['id'], post['id']))
			thread['posts'] = posts
			thread['max_post_id'] = posts[-1]['id']
			thread['num_posts_omitted'] = (posts[1]['id']-1) if (len(posts) > 1) else 0
			thread['num_files_omitted'] = (thread['num_files'] - files_shown)
		self.commit()
		return {'board': self.board, 'threads': threads, 'cur_page': page, 'pages': pages, 'new_post': {'bump': True}, 'new_thread': {}, 'on_thread_page': False}
	
	
	def get_view_thread_data(self, id):
		self.begin()
		#get thread data
		thread = self.select_1("SELECT * FROM threads WHERE id = ?", id)
		if thread == None:
			self.commit()
			return None
		#get forked post data?
		if thread['forked_thread_id'] is not None and thread['forked_post_id'] is not None:
			thread['forked_post'] = self.get_forked_post_data(thread['forked_thread_id'], thread['forked_post_id'], id)
		else:
			thread['forked_post'] = None
		#get post data
		q = """	SELECT thread_id, id, format_time(posted) AS posted, name, tripcode, mod, format_status(status) AS status, comment, file_meta,
				EXISTS (SELECT * FROM threads WHERE forked_thread_id = posts.thread_id AND forked_post_id = posts.id) AS has_forked_threads
				FROM posts
				WHERE thread_id = ?
				ORDER BY id"""
		q2 =  """	SELECT id, subject
					FROM threads
					WHERE forked_thread_id = ? AND forked_post_id = ?
					ORDER BY id"""
		posts = self.select(q, id)
		thread['blurb'] = parser.blurb(self.board, id, thread['subject'], posts[0]['comment'])
		#parse for display
		for post in posts:
			post['file_meta'] = cPickle.loads(str(post['file_meta'])) if post['file_meta'] else None
			post['comment'] = parser.parse(post['comment'], self.board, (id, post['id'], True))
			#get list of threads forked from this post
			if post['has_forked_threads']:
				post['forked_threads'] = self.select(q2, (id, post['id']))
		thread['posts'] = posts
		thread['num_posts'] = len(posts)
		thread['max_post_id'] = posts[-1]['id']
		self.commit()
		return {'board': self.board, 'thread': thread, 'new_post': {'bump': True}, 'on_thread_page': True}
	
	
	def get_list_data(self):
		q = """	SELECT *,
				format_time(updated) AS updated,
				(SELECT comment FROM posts WHERE thread_id = threads.id AND posts.id = 0) AS head_comment,
				(SELECT COUNT(*) FROM posts WHERE thread_id = threads.id) AS num_posts,
				(SELECT COUNT(*) FROM posts WHERE thread_id = threads.id AND file_md5 IS NOT NULL) AS num_files
				FROM threads
				ORDER BY sticky DESC, updated DESC"""
		threads = self.select(q)
		num_pages = int(math.ceil(len(threads) / float(self.board.THREADS_PER_PAGE)))
		pages = range(max(1, num_pages))
		#
		for thread in threads:
			thread['blurb'] = parser.blurb(self.board, thread['id'], thread['subject'], thread['head_comment'])
		return {'board': self.board, 'threads': threads, 'cur_page': -1, 'pages': pages, 'new_thread': {}}


	def get_new_reply_data(self, thread_id):
		q = """	SELECT id, locked,
				(SELECT MAX(id)+1 FROM posts WHERE thread_id = threads.id) AS next_id
				FROM threads
				WHERE id = ?"""
		thread = self.select_1(q, thread_id)
		return None if (thread is None) else {'thread': thread}
	
	
	def add_new_thread(self, subject, sticky, locked, forked_thread_id, forked_post_id, name, tripcode, mod, comment, file_md5, file_meta, ip):
		""" returns the id of the thread """
		
		self.begin(immediate = True)
		q = """INSERT INTO threads (subject, sticky, locked, updated, forked_thread_id, forked_post_id) VALUES (?, ?, ?, inow(), ?, ?)"""
		self.insert(q, (subject, sticky, locked, forked_thread_id, forked_post_id))
		thread_id = self.cur.lastrowid
		
		# insert into posts
		file_md5 = buffer(file_md5) if file_md5 else None
		file_meta = buffer(cPickle.dumps(file_meta, True)) if file_meta else None
		q = """	INSERT INTO posts (thread_id, id, posted, status, name, tripcode, mod, comment, file_md5, file_meta, ip)
				VALUES (?, 0, inow(), 0, ?, ?, ?, ?, ?, ?, ?)"""
		self.insert(q, (thread_id, name, tripcode, mod, comment, file_md5, file_meta, ip))
		self.commit()
		
		#delete old threads
		self.begin(immediate = True)
		if self.board.DELETE_FORKED_THREADS:
			q = """	SELECT id FROM threads
					ORDER BY sticky DESC, updated DESC
					LIMIT ?, -1"""
		else:
			q = """	SELECT id FROM threads
					WHERE id NOT IN (select forked_thread_id FROM threads)
					ORDER BY sticky DESC, updated DESC
					LIMIT ?, -1"""
		dt = self.select(q, self.board.MAX_THREADS)
		for t in dt:
			self.delete_thread(t['id'], transaction = False)
		self.commit()
		
		return thread_id
	

	def edit_thread(self, thread_id, subject, sticky, lock):
		q = "UPDATE threads SET sticky = ?, locked = ?, subject = ? WHERE id = ?"
		self.insert(q, (sticky, lock, subject, thread_id))
	
	
	def delete_thread(self, thread_id, transaction = True):
		if transaction:
			self.begin()
		self.insert("DELETE FROM threads WHERE id = ?", thread_id)
		#delete files
		q = "SELECT id, file_meta FROM posts WHERE thread_id = ? AND file_md5 IS NOT NULL"
		files = self.select(q, thread_id)
		for file in files:
			uploaded_files.delete(self.board, thread_id, file['id'], cPickle.loads(str(file['file_meta'])))
		try:
			util.rmdirs(os.path.join(self.board.get_file_dir(), '%d' % thread_id))
		except:
			pass
		self.insert("DELETE FROM posts WHERE thread_id = ?", thread_id)
		self.insert("DELETE FROM flags WHERE thread_id = ?", thread_id)
		self.insert("DELETE FROM reviewed_posts WHERE thread_id = ?", thread_id)
		if transaction:
			self.commit()
	
		
	def edit_post(self, thread_id, post_id, status, name, tripcode, mod, comment, delete_file, reviewed):
		self.begin(immediate = True)
		q = "SELECT file_md5, file_meta FROM posts WHERE thread_id = ? AND id = ?"
		row = self.select_1(q, (thread_id, post_id))
		if row:
			if delete_file:
				file_meta = cPickle.loads(str(row['file_meta']))
				uploaded_files.delete(self.board, thread_id, post_id, file_meta)
				q = "UPDATE posts SET file_md5 = NULL, file_meta = NULL WHERE thread_id = ? AND id = ?"
				self.insert(q, (thread_id, post_id))
			q = "UPDATE posts SET status = ?, name = ?, tripcode = ?, mod = ?, comment = ? WHERE thread_id = ? AND id = ?"
			self.insert(q, (POSTS_STATUS.index(status), name, tripcode, int(mod), comment, thread_id, post_id))
			if reviewed:
				q = "INSERT OR IGNORE INTO reviewed_posts (thread_id, post_id, time) VALUES (?, ?, inow())"
				self.insert(q, (thread_id, post_id))
		self.commit()

	
	def delete_post(self, thread_id, post_id):
		self.begin(immediate = True)
		q = "SELECT file_meta FROM posts WHERE thread_id = ? AND id = ?"
		data = self.select_1(q, (thread_id, post_id))
		if data:
			if data['file_meta']:
				uploaded_files.delete(self.board, thread_id, post_id, cPickle.loads(str(data['file_meta'])))
			q = """	UPDATE posts
					SET status = 1, tripcode = '', name = '', comment = '', ip = NULL, file_md5 = NULL, file_meta = NULL
					WHERE thread_id = ? AND id = ?"""
			self.insert(q, (thread_id, post_id))
		self.commit()


	def add_new_reply(self, thread_id, name, tripcode, mod, comment, file_md5, file_meta, ip, bump, lock):
		""" returns the id of the post or None on error """
		self.begin(immediate = True)
		q = "SELECT MAX(id)+1 AS next_id FROM posts WHERE thread_id = ?"
		row = self.select_1(q, thread_id)
		if not row['next_id'] or row['next_id'] >= self.board.MAX_THREAD_POSTS:
			self.commit()
			return None
		file_md5 = buffer(file_md5) if file_md5 else None
		file_meta = buffer(cPickle.dumps(file_meta, True)) if file_meta else None
		q = """	INSERT INTO posts (thread_id, id, status, posted, name, tripcode, mod, comment, file_md5, file_meta, ip)
				VALUES (?, ?, 0, inow(), ?, ?, ?, ?, ?, ?, ?)"""
		self.insert(q, (thread_id, row['next_id'], name, tripcode, mod, comment, file_md5, file_meta, ip))
		if bump:
			self.bump(thread_id)
		if lock:
			self.insert("UPDATE threads SET locked = 1 WHERE id = ?", thread_id)
		self.commit()
		return row['next_id']


	def flag_post(self, thread_id, post_id, ip):
		self.begin(immediate = True)
		#only flag posts that exist
		q = "SELECT 1 FROM posts WHERE thread_id = ? AND id = ?"
		if self.select_1(q, (thread_id, post_id)):
			#only flag post if not already reviewed
			q = "SELECT 1 FROM reviewed_posts WHERE thread_id = ? AND post_id = ?"
			if not self.select_1(q, (thread_id, post_id)):
				q = "INSERT OR IGNORE INTO flags (thread_id, post_id, ip) VALUES (?, ?, ?)"
				self.insert(q, (thread_id, post_id, ip))
		self.commit()


	def get_post_options_data(self, thread_id, post_id, ip, correct_cookie, msg = None):
		q = """	SELECT format_status(status) as status, thread_id, id AS post_id, tripcode,
				EXISTS (SELECT 1 FROM reviewed_posts WHERE reviewed_posts.thread_id = posts.thread_id AND post_id = posts.id) AS reviewed,
				EXISTS (SELECT 1 FROM flags WHERE flags.thread_id = posts.thread_id AND post_id = posts.id AND flags.ip = ?) AS flagged
				FROM posts
				WHERE thread_id = ? AND id = ?"""
		data = self.select_1(q, (ip, thread_id, post_id))
		if data is None:
			return None
		data['board'] = self.board
		data['correct_cookie'] = correct_cookie
		data['msg'] = msg
		return data
	

	def get_post_mod_options_data(self, thread_id, post_id, general_db, msg = None):
		try:
			self.begin()
			q = "SELECT id, subject, sticky, locked FROM threads WHERE id = ?"
			thread = self.select_1(q, thread_id)
			if thread == None:
				return None
			q = """	SELECT thread_id, id, comment, name, tripcode, mod, ip, fqdn(ip) as fqdn, format_status(status) as status, file_meta,
					(SELECT COUNT(*) FROM flags WHERE flags.thread_id = posts.thread_id AND post_id = posts.id) AS num_flags,
					EXISTS (SELECT * FROM reviewed_posts WHERE reviewed_posts.thread_id = posts.thread_id AND reviewed_posts.post_id = posts.id) AS reviewed
					FROM `posts` WHERE thread_id = ? AND id = ?"""
			post = self.select_1(q, (thread_id, post_id))
			if post == None:
				return None
			if post['file_meta']:
				post['file_meta'] = cPickle.loads(str(post['file_meta']))
			ban = general_db.get_ban(post['ip'])
			return {'board': self.board, 'post': post, 'thread': thread, 'ban': ban, 'msg': msg}
		finally:
			self.commit()
	

	def file_exists(self, file_md5):
		return self.select_1("SELECT 1 FROM posts WHERE file_md5 = ?", buffer(file_md5)) is not None
	

	def post_exists(self, thread_id, post_id):
		return self.select_1("SELECT 1 FROM posts WHERE thread_id = ? AND id = ?", (thread_id, post_id)) is not None


	def post_not_deleted(self, thread_id, post_id):
		return self.select_1("SELECT 1 FROM posts WHERE thread_id = ? AND id = ? AND status = 0", (thread_id, post_id)) is not None

	
	def get_post_file_md5(self, thread_id, post_id):
		data = self.select_1("SELECT file_md5 FROM posts WHERE thread_id = ? AND id = ?", (thread_id, post_id))
		return str(data['file_md5']) if data else None


	def get_post_tripcode(self, thread_id, post_id):
		q = "SELECT tripcode FROM posts WHERE thread_id = ? AND id = ?"
		row = self.select_1(q, (thread_id, post_id))
		return row['tripcode'] if row else None
	

	def is_post_flood(self, ip):
		q = "SELECT 1 FROM posts WHERE ip = ? AND posted > ? LIMIT 1"
		return self.select_1(q, (ip, time.time() - self.board.MIN_SECONDS_BETWEEN_POSTS)) is not None


	def is_thread_flood(self, ip):
		q = "SELECT 1 FROM posts WHERE ip = ? AND id = 0 AND posted > ? LIMIT 1"
		return self.select_1(q, (ip, time.time() - self.board.MIN_SECONDS_BETWEEN_THREADS)) is not None


	def bump(self, thread_id):
		q = "UPDATE threads SET updated = inow() WHERE id = ?"
		self.insert(q, thread_id)
	
	
	def anonymize_old_posts(self):
		q = "UPDATE posts SET ip = NULL WHERE posted < ?"
		self.insert(q, time.time() - self.board.DAYS_TO_KEEP_POST_IPS*24*60*60)


	def get_mod_area_overview_data(self):
		self.begin()
		# get a list of flagged posts
		q = """	SELECT thread_id, post_id, COUNT(*) AS count
				FROM flags
				WHERE NOT EXISTS (SELECT * FROM reviewed_posts WHERE reviewed_posts.thread_id = flags.thread_id AND reviewed_posts.post_id = flags.post_id)
				GROUP BY thread_id, post_id
				ORDER BY count DESC, thread_id, post_id"""
		flags = self.select(q)
		# get a list of reviewed posts
		q = """ SELECT posts.thread_id, post_id, format_time(time) AS time, format_status(status) AS status
				FROM reviewed_posts, posts
				WHERE posts.thread_id = reviewed_posts.thread_id AND posts.id = post_id
				ORDER BY reviewed_posts.time DESC"""
		reviewed_posts = self.select(q)
		self.commit()
		return {'flags': flags, 'reviewed_posts': reviewed_posts, 'stats': self.get_stats()}


	def get_stats(self):
		q = """SELECT
			(SELECT COUNT(*) FROM threads) AS num_threads,
			(SELECT COUNT(*) FROM posts) AS num_posts,
			(SELECT COUNT(*) FROM posts WHERE file_md5 IS NOT NULL) AS num_files"""
		r = self.select_1(q)
		r['uploaded_files_size'] = util.get_dir_size(self.board.get_file_dir())
		return r
