""" for converting user comment input to displayable html """

import re
from util import clamp_str
import config, util, cgi2

URL_REGEX = {}
LONG_WORD_REGEX = {}
for board in config.BOARDS:
	if board.AUTO_LINK_URLS:
		URL_REGEX[board] = re.compile(r'(?:www\.|(?:(?:%s)://))(?:(?:%%[0-9a-fA-F]{2})|[0-9a-zA-Z;/\?:@&\=\+\$#\-_\.~\*!\'])+' % r'|'.join(board.AUTO_LINK_URLS))
	else:
		URL_REGEX[board] = re.compile(r'$_^')
	LONG_WORD_REGEX[board] = re.compile(r'\S{%d}(?=\S)' % board.MAX_COMMENT_WORD_LENGTH)

YOUTUBE_REGEX = re.compile(r'^http://(?:(?:www|[a-z]{2}).)?youtube\.com/watch\?.*v=(?P<id>[-a-z0-9_]{11})', re.IGNORECASE)
MAIL_REGEX = re.compile(r'[a-z0-9._%+-]+@[a-z0-9-]+(?:\.[a-z0-9-]+)+')
CHAIN_REGEX = re.compile(r'>>\d+')


class Parser:
	def __init__(self, board, context):
		self.board = board
		self.context = context
		self.youtubes = 0
	
	def parse(self, input):
		input_len = len(input)
		output = []
		first = True
		for line in input.splitlines():
			if not first:
				output.append('<br>')
			else:
				first = False
			if line.startswith('>'):
				output.append('<span class="quote">')
				self.parse_sub(line, output, False)
				output.append('</span>')
			else:
				self.parse_sub(line, output, True)
		output = ''.join(output)
		return ('<div class="emote">%s</div>' % output) if (input_len <= self.board.MAX_EMOTE_LENGTH) else output


	def parse_sub(self, input, output, embed_youtubes):
		for m in util.re_msplit(URL_REGEX[self.board], input):
			if isinstance(m, basestring):
				self.parse_sub2(m, output)
			else:
				link = m.group()
				if link.startswith('www.'):
					link = 'http://' + link
				ym = YOUTUBE_REGEX.search(link)
				if ym and embed_youtubes and self.youtubes < self.board.MAX_YOUTUBE_EMBEDS_PER_POST:
					id = ym.group('id')
					output.append('<div class="youtube"><object width="425" height="344"><param name="movie" value="http://www.youtube.com/v/%s&hl=en"></param><embed src="http://www.youtube.com/v/%s&hl=en" type="application/x-shockwave-flash" width="425" height="344"></embed></object></div>' % (id, id))
					self.youtubes += 1
				else:
					output.append('<a class="posted_url" href="%s">%s</a>' % (cgi2.escape(link), cgi2.escape(clamp_str(link, self.board.MAX_COMMENT_WORD_LENGTH, '...'))))


	def parse_sub2(self, input, output):
		for m in util.re_msplit(MAIL_REGEX, input):
			if isinstance(m, basestring):
				self.parse_sub3(m, output)
			else:
				mail = m.group()
				output.append('<a class="posted_url" href="mailto:%s">%s</a>' % (cgi2.escape(mail), cgi2.escape(clamp_str(mail, self.board.MAX_COMMENT_WORD_LENGTH, '...'))))


	def parse_sub3(self, input, output):
		for m in util.re_msplit(CHAIN_REGEX, input):
			if isinstance(m, basestring):
				self.parse_sub4(m, output)
			else:
				to = int(m.group()[2:])
				if (self.context is None) or (to >= self.context[1]):
					output.append('<span class="invalid_chain">&gt;&gt;%d</span>' % to)
				elif self.context[2]:	# on thread page
					output.append('<a class="chain" href="#%d" onclick="return chain(null,%s,%d)">&gt;&gt;%d</a>' % (to, self.context[1], to, to))
				else:
					output.append('<a class="chain" href="/%s/%s#%d" onclick="return chain(%s,%s,%d)">&gt;&gt;%d</a>' % (self.board.DIR, self.context[0], to, self.context[0], self.context[1], to, to))
				
	
	def parse_sub4(self, input, output):
		for m in util.re_msplit(LONG_WORD_REGEX[self.board], input):
			if isinstance(m, basestring):
				output.append(cgi2.escape(m))
			else:
				output.append(cgi2.escape(m.group()) + ' ')


def parse(input, board, context = None):
	""" context is None or (thread_id, post_id, on_thread_page?) """
	return Parser(board, context).parse(input)


def blurb(board, thread_id, subject, head_comment):
	""" the blurb for a thread is what is displayed on /list """
	if subject:
		return subject
	if head_comment:
		head_comment = re.sub(r'(\r\n)+', ' ', head_comment)
		return util.clamp_str(head_comment, board.MAX_SUBJECT_LENGTH, '...')
	return 'Thread %d' % thread_id
