""" stuff not tied to this project """

import re, hashlib, os, os.path, random, cStringIO, StringIO, time, tempfile, socket


HEX_REGEX = re.compile('^[a-zA-Z0-9]*$')

AMPM_HOURS = ['12:%.2dam'] \
	+ ['%d:%%.2dam' % i for i in xrange(1, 12)] \
	+ ['12:%.2dpm'] \
	+ ['%d:%%.2dpm' % i for i in xrange(1, 12)]


class Timer:
	""" used to calc the difference in time between two points in time """
	
	def __init__(self, start):
		self.start = start

	def calc(self):
		return time.time() - self.start


class TempFile:
	""" holds a file name that will be unlinked when the instance is garbage collected """
	
	def __init__(self):
		self.path = tempfile.mkstemp()[1]

	def __del__(self):
		try:
			os.unlink(self.path)
		except:
			pass


class ObjDict(dict):
	def __init__(self, **d):
		self.update(d)

	def __getattr__(self, name):
		return self[name]
	
	def __setattr__(self, name, value):
		self[name] = value


def is_hex(s):
	return HEX_REGEX.match(s) is not None


def unhex(input):
	out = ''
	i = 0
	while i+1 < len(input):
		out += chr(int(input[i:i+2], 16))
		i += 2
	return out


def valid_ip(input):
	try:
		socket.inet_aton(input)
		return True
	except:
		return False

def rnd_ip():
	return '.'.join([str(random.randint(0, 255)) for x in xrange(4)])


def pad_ip(input):
	""" pads an ip so it is of the form xxx.xxx.xxx.xxx """
	return '.'.join([x.rjust(3, '0') for x in input.split('.')])


def valid_time(timestamp):
	try:
		time.gmtime(timestamp)
		return True
	except:
		return False


def sint(input, default = 0):
	""" "safe" int cast """
	try:
		return int(input)
	except ValueError:
		return default	

def psint(input, default = 0):
	""" positive "safe" int cast """
	return abs(sint(input, default))


def sfloat(input, default = 0):
	""" "safe" float cast """
	try:
		return float(input)
	except ValueError:
		return default

def psfloat(input, default = 0):
	""" positive "safe" float cast """
	return abs(sfloat(input, default))


def md5(input, hex = True):
	""" one-call md5 """
	if isinstance(input, unicode):
		input = input.encode('utf-8', 'replace')
	h = hashlib.md5()
	h.update(input)
	return h.hexdigest() if hex else h.digest()


def sha1(input, hex = True):
	""" one-call sha1 """
	if isinstance(input, unicode):
		input = input.encode('utf-8', 'replace')
	h = hashlib.sha1()
	h.update(input)
	return h.hexdigest() if hex else h.digest()


def rand_str(len, pos = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'):
	return ''.join([random.choice(pos) for i in xrange(len)])


def clamp_str(input, max_len, app = ''):
	if not isinstance(input, basestring):
		return ''
	if (len(input) <= max_len):
		return input
	elif len(app) < max_len:
		return input[:max_len - len(app)] + app
	else:
		return app[:max_len]


def limit(input, *limits):
	if input in limits:
		return input
	return limits[0]


def get_limited(dict, key, limits, index = 0):
	if key in dict:
		return limit(dict[key], limits, index)
	return limits[index]


def clamp(val, min, max):
	"""	sint's val, then clamps it between min and max """
	val = sint(val)
	if val > max:
		return max
	if val < min:
		return min
	return val


def pad(seq, length, value = ''):
	""" pads a list with empty values so it is at least length in size """
	return seq if (len(seq) >= length) else list(seq) + [value]*(length-len(seq))


def remove_empties(seq):
	return [x for x in seq if x != '']


def tuplize(x):
	return x if isinstance(x, tuple) else (x,)


def file_get_contents(file):
	""" one call file read """
	f = open(file, 'rb')
	c = f.read()
	f.close()
	return c


def file_set_contents(file, contents):
	""" one call file write """
	f = open(file, 'wb')
	f.write(contents)
	f.close()


def quiet_unlink(file):
	try:
		os.unlink(file)
	except:
		pass


def copy_file(path, file):
	""" copies data from file object file into the physical file at path. returns the md5 of the data copied """
	h = hashlib.md5()
	rt = open(path, 'wb')
	c = file.read(4096)
	while c:
		h.update(c)
		rt.write(c)
		c = file.read(4096)
	file.close()
	rt.close()
	return h.digest()


def get_dir_size(path):
	""" recursive directory size """
	size = 0
	for p, d, files in os.walk(path):
		for f in files:
			size += os.path.getsize(os.path.join(p, f))
	return size


def rmdirs(path):
	""" recursive directory delete """
	for root, dirs, files in os.walk(path, topdown = False):
		for file in files:
			os.unlink(os.path.join(root, file))
		for dir in dirs:
			os.rmdir(os.path.join(root, dir))
	os.rmdir(path)


def sanitize_file_name(input):
	if input.endswith('~'):
		input = input[:-1]
	input = re.sub(r'^[\.\-]|[^a-zA-Z0-9\-=\+_\.\[\]\(\)]', '_', input)
	input = re.sub(r'_+', '_', input)
	input = re.sub(r'\.+', '.', input)
	return input[:128]


def gzip_encode(bin_str):
	""" one call gzip encode """
	import gzip
	f = cStringIO.StringIO()
	g = gzip.GzipFile(mode = 'wb', compresslevel = 5, fileobj = f)
	g.write(bin_str)
	g.close()
	return f.getvalue()


def gzip_decode(bin_str):
	""" one call gzip decode """
	import gzip
	f = cStringIO.StringIO(bin_str)
	g = gzip.GzipFile(mode = 'rb', fileobj = f)
	s = g.read()
	g.close()
	return s


def format_file_size(size):
	size = float(size)
	if size < 1024:
		return '%db' % size
	size = size/1024	#now in kb
	if size < 1024:
		return '%dkb' % size
	size = size/1024	#now in mb
	if size < 1024:
		return '%dmb' % size
	size = size/1024	#now in gb
	return '%dgb' % size



def ampmtime(struct_time):
	return AMPM_HOURS[struct_time.tm_hour] % struct_time.tm_min


def format_time(timestamp = None, gmt_offset = 0):
	now = int(time.time())
	if timestamp is None:
		timestamp = now
	
	now_st = time.gmtime(now + gmt_offset*60*60)
	then_st = time.gmtime(timestamp + gmt_offset*60*60)
	apt = ampmtime(then_st)
	
	if (timestamp <= now) and (now_st.tm_year == then_st.tm_year):
		if now_st.tm_yday == then_st.tm_yday:
			sec_dif = now-timestamp
			if sec_dif == 1:
				return apt + ' - 1 second ago'
			elif sec_dif < 60:
				return apt + ' - %d seconds ago' % sec_dif
			min_dif = sec_dif/60
			if min_dif == 1:
				return apt + ' - 1 minute ago'
			elif min_dif < 60:
				return apt + ' - %d minutes ago' % min_dif
			hr_dif = min_dif/60
			if hr_dif == 1:
				return apt + ' - 1 hour ago'
			elif hr_dif < 24:
				return apt + ' - %d hours ago' % hr_dif
		if now_st.tm_yday-1 == then_st.tm_yday:
			return time.strftime('%b %d - Yesterday - ', then_st) + apt
		else:
			return time.strftime('%b %d ', then_st) + apt
	
	return time.strftime('%b %d %Y ', then_st) + apt


def re_msplit(regex, input):
	""" a regex split that returns a list of interspersed strings and match objects """
	output = []
	start = 0
	for mo in regex.finditer(input):
		if mo.start() > start:
			output.append(input[start:mo.start()])
		output.append(mo)
		start = mo.end()
	if start != len(input):
		output.append(input[start:])
	return output