""" every function in this module returns a specific page """


import config, common, parser, util, database, uploaded_files, cookies, sessions, tripcodes, json
from cgi2 import Request, Response, RedirectResponse
from util import get_limited, clamp_str, limit, sint, psint, valid_ip
from common import Img2Exception, NotFound, Forbidden, View, render_to_response, render_to_string, is_mod, get_mod_name
from database import GeneralDataBase


def transload(request, board):
	if request.POST('pw') != util.md5('transload' + config.UNIQUE_STRING):
		raise Forbidden

	import urllib, cStringIO, socket, cgi2, random, os
	
	db = board.db()
	general_db = GeneralDataBase()

	socket.setdefaulttimeout(15)
	for i, comment in enumerate(request.post_vars['comment']):
		image = request.POST('img', index = i)
		file_md5, file_meta, temp_file = None, None, None
		if image:
			try:
				file = cStringIO.StringIO(urllib.urlopen(image).read())
				file_name = os.path.basename(image)
				file_md5, file_meta, temp_file = uploaded_files.check(cgi2.UploadedFile(file_name, file), board, db, general_db)
			except:
				pass
		if i == 0:
			thread_id = db.add_new_thread('', False, False, None, None, '', None, False, comment, file_md5, file_meta, '127.0.0.1')
			post_id = 0
		else:
			post_id = db.add_new_reply(thread_id, '', None, False, comment, file_md5, file_meta, '127.0.0.1', True, False)
		if file_meta:
			uploaded_files.save(temp_file, board, thread_id, post_id, file_meta)
	return RedirectResponse('/%s/%s' % (board.DIR, thread_id))


def feed_boards(request):
	data = [{'title': b.TITLE, 'href': 'http://%s/%s' % (request['SERVER_NAME'], b.DIR)} for b in config.BOARDS]
	return Response(json.encode(data), 'application/json; charset=utf-8')


def confirm_overage(request):
	r = RedirectResponse(request['HTTP_REFERER'] or '/')
	cookies.set_overage(r)
	return r


def index(request):
	return RedirectResponse('/%s/' % config.BOARDS[0].DIR)


def view_board(request, board, page):
	page = int(page)
	data = board.db().get_view_board_data(page)
	if data is None:
		raise NotFound
	else:
		return render_to_response(request, 'view_board.html', data)


def view_thread(request, board, id):
	id = int(id)
	data = board.db().get_view_thread_data(id)
	if not data:
		raise NotFound
	else:
		return render_to_response(request, 'view_thread.html', data)


def list(request, board):
	data = board.db().get_list_data()
	return render_to_response(request, 'list.html', data)


def new_thread(request, board):
	
	if request.method == 'GET':
		forked_thread_id = request.GET('t')
		forked_post_id = request.GET('p')
		return render_to_response(request, 'new_thread.html', {'board': board, 'new_thread': {'forked_thread_id': forked_thread_id, 'forked_post_id': forked_post_id}})
	
	elif request.method != 'POST' or not request.local_referer():
		raise Forbidden
	
	sent = {
		'name':					clamp_str(request.POST('name', '').strip(), board.MAX_NAME_LENGTH),
		'tripcode':				request.POST('tripcode', ''),
		'subject':				clamp_str(request.POST('subject', '').strip(), board.MAX_SUBJECT_LENGTH),
		'comment':				request.POST('comment', '').strip(),
		'file':					request.FILE('file'),
		'forked_thread_id':		request.POST('forked_thread_id'),
		'forked_post_id':		request.POST('forked_post_id'),
		'noko': 				'noko' in request.post_vars,
		'ip':					request['REMOTE_ADDR'],
		'sticky':				'sticky' in request.post_vars,
		'lock':					'lock' in request.post_vars,
		'mod':					'mod' in request.post_vars
	}
	
	def set_cookies(h):
		cookies.set_noko(h, sent['noko'])
		cookies.set_name_and_tripcode(h, sent['name'], sent['tripcode'])
	
	def error(msg):
		h = render_to_response(request, 'new_thread.html', {'board': board, 'error_msg': msg, 'new_thread': sent})
		set_cookies(h)
		return h
	
	boardDB = board.db()
	general_db = GeneralDataBase()
	
	ban = general_db.get_ban(sent['ip'])
	if ban:
		h = render_to_response(request, 'new_thread.html', {'board': board, 'ban': ban, 'new_thread': sent})
		set_cookies(h)
		return h
	
	if len(sent['comment']) == 0 and not (board.ALLOW_NO_COMMENT_WITH_IMAGE and sent['file']):
		return error('You must write something.')
	
	if len(sent['comment']) > board.MAX_COMMENT_LENGTH:
		return error('Comment too long, max allowed length is %d characters.' % board.MAX_COMMENT_LENGTH)
	
	mod = get_mod_name(request)
	
	#enforce board rules, but not for mods
	if not mod:
		sent['sticky'] = sent['lock'] = sent['mod'] = False
	
		if sent['file'] is None and not board.ALLOW_THREAD_WITHOUT_FILE:
			return error('You must supply a file/image to start a thread.')
			
		if boardDB.is_thread_flood(sent['ip']):
			return error('You are making threads too fast. Slow down!')
	
		if not board.ALLOW_NAMES:		sent['name'] == ''
		if not board.ALLOW_TRIPCODES:	sent['tripcode'] = ''

	#check forked post is valid - if set
	if board.FORKING and (sent['forked_thread_id'] is not None) and (sent['forked_post_id'] is not None):
		forked_thread_id = psint(sent['forked_thread_id'])
		forked_post_id = psint(sent['forked_post_id'])
		if forked_post_id == 0:
			return error('You can\'t fork the first post.')
		if not boardDB.post_exists(forked_thread_id, forked_post_id):
			return error('You can\'t fork a non existent post.')
	else:
		forked_thread_id = forked_post_id = None
		
	if board.NOKO == 'never':		sent['noko'] = False
	elif board.NOKO == 'always':	sent['noko'] = True
	if not board.VALID_FILE_TYPES:	sent['file'] = None
	
	if sent['file']:
		try:
			file_md5, file_meta, temp_file = uploaded_files.check(sent['file'], board, boardDB, general_db)
		except Img2Exception, e:
			return error(str(e))
	else:
		file_md5, file_meta = None, None
	
	name = sent['name'] or board.get_default_name()
	thread_id = boardDB.add_new_thread(sent['subject'], sent['sticky'], sent['lock'], forked_thread_id, forked_post_id, name, tripcodes.make(board, sent['tripcode']), sent['mod'], sent['comment'], file_md5, file_meta, sent['ip'])
	
	if file_md5:
		uploaded_files.save(temp_file, board, thread_id, 0, file_meta)
	
	path = '/%s/' % board.DIR
	if sent['noko']:
		path += str(thread_id)
	h = RedirectResponse(path)
	set_cookies(h)
	cookies.set_post_verifier(h, board, thread_id, 0)	
	return h


def new_reply(request, board, thread_id):
	thread_id = int(thread_id)
	
	if request.method == 'GET':
		qid = request.GET('q')
		comment = '' if (qid is None) else ('>>%d' % util.psint(qid))
		return render_to_response(request, 'new_reply.html', {'board': board, 'thread': {'id': thread_id}, 'new_post': {'comment': comment}})
	
	elif request.method != 'POST' or not request.local_referer():
		raise Forbidden
	
	sent = {
		'name': 				clamp_str(request.POST('name', '').strip(), board.MAX_NAME_LENGTH),
		'tripcode':				request.POST('tripcode', ''),
		'comment':				request.POST('comment', '').strip(),
		'file':					request.FILE('file'),
		'noko': 				'noko' in request.post_vars,
		'bump':					'bump' in request.post_vars,
		'lock':					'lock' in request.post_vars,
		'mod':					'mod' in request.post_vars,
		'ip':					request['REMOTE_ADDR']
	}
	
	general_db = GeneralDataBase()
	boardDB = board.db()
	
	data = boardDB.get_new_reply_data(thread_id)
	if data is None:
		raise NotFound
	thread = data['thread']

	def set_cookies(h):
		cookies.set_noko(h, sent['noko'])
		cookies.set_name_and_tripcode(h, sent['name'], sent['tripcode'])
	
	def error(msg):
		h = render_to_response(request, 'new_reply.html', {'board': board, 'error_msg': msg, 'thread': thread, 'new_post': sent})
		set_cookies(h)
		return h
	
	ban = general_db.get_ban(sent['ip'])
	if ban:
		h = render_to_response(request, 'new_reply.html', {'board': board, 'ban': ban, 'thread': thread, 'new_post': sent})
		set_cookies(h)
		return h
	
	if thread['next_id'] >= board.MAX_THREAD_POSTS:
		return error('This thread is full.')

	if len(sent['comment']) == 0 and not (board.ALLOW_NO_COMMENT_WITH_IMAGE and sent['file']):
		return error('You must write something.')
	
	if len(sent['comment']) > board.MAX_COMMENT_LENGTH:
		return error('Comment too long, max allowed length is %s characters.' % board.MAX_COMMENT_LENGTH)
	
	mod = get_mod_name(request)
	
	# enforce board rules, but not for mods
	if not mod:
		sent['lock'] = sent['mod'] = False
		
		if thread['locked']:
			return error('This thread is locked.')
	
		if sent['file'] is None and not board.ALLOW_REPLY_WITHOUT_FILE:
			return error('You must supply a file/image with a reply.')
		
		if boardDB.is_post_flood(sent['ip']):
			return error('You are posting too fast. Relax!')
		
		if not board.ALLOW_NAMES:		sent['name'] == ''
		if not board.ALLOW_TRIPCODES:	sent['tripcode'] = ''		

	if board.NOKO == 'never':		sent['noko'] = False
	elif board.NOKO == 'always':	sent['noko'] = True
	if board.BUMP == 'never':		sent['bump'] = False
	elif board.BUMP == 'always':	sent['bump'] = True
	if not board.VALID_FILE_TYPES:	sent['file'] = None
	
	if sent['file']:
		try:
			file_md5, file_meta, temp_file = uploaded_files.check(sent['file'], board, boardDB, general_db)
		except Img2Exception, e:
			return error(str(e))
	else:
		file_md5, file_meta = None, None
	
	name = sent['name'] or board.get_default_name()
	post_id = boardDB.add_new_reply(thread_id, name, tripcodes.make(board, sent['tripcode']), sent['mod'], sent['comment'], file_md5, file_meta, sent['ip'], sent['bump'], sent['lock'])
	if post_id == None:
		return error('Error adding reply.')
	
	if file_md5:
		uploaded_files.save(temp_file, board, thread_id, post_id, file_meta)
	
	path = '/%s/' % board.DIR
	if sent['noko']:
		path += '%d#%d' % (thread_id, post_id)
	h = RedirectResponse(path)
	set_cookies(h)
	cookies.set_post_verifier(h, board, thread_id, post_id)
	return h


def mod_area(request, section):
	
	if not get_mod_name(request):
		#may be logging in this request
		name = request.POST('name')
		password = request.POST('password')
		if is_mod(name, password):
			sessions.set_mod_login(request, name, password)
			if request.POST('return'):
				return RedirectResponse(request.POST('referer', '/'))
		else:
			sessions.delete_mod_login(request)
			return render_to_response(request, 'mod_login.html', {'section': section, 'referer': request['HTTP_REFERER']})
	
	database.cleanup()
	action = request.POST('action')
	if not request.local_referer():
		action = None
	msg = None
	
	if section == 'overview':
		data = database.get_mod_area_overview_data()
		data['section'] = section
		data['msg'] = msg
		return render_to_response(request, 'mod_overview.html', data)
	
	elif section == 'bans':
		
		if action:
			action = limit(action, 'add', 'delete')
			
			if action == 'add':
				import time
				now = int(time.time())
				ip = request.POST('ip', '').strip()
				expires_parsed = common.parse_time(request.POST('expires', '').strip())
				reason = request.POST('reason').strip()
				
				if expires_parsed is None:
					msg = 'Invalid expiry time'
				elif now + expires_parsed > (1<<31)-1:
					msg = 'Expiry time too far in future'
				elif not util.valid_ip(ip):
					msg = 'Invalid IP'
				else:
					GeneralDataBase().add_ban(ip, now + expires_parsed, reason)
					msg = 'Ban Added'
			
			elif action == 'delete':
				ip = request.POST('ip', '')
				GeneralDataBase().delete_ban(ip)
				msg = 'Ban Deleted.'
		
		filter = request.GET('filter', '')
		data = GeneralDataBase().get_mod_area_bans_data(filter)
		data['new_ban_ip'] = request.GET('new_ban_ip', '')
		data['filter'] = filter
		data['section'] = section
		data['msg'] = msg
		return render_to_response(request, 'mod_bans.html', data)
	
	elif section == 'files':
	
		if action:
			action = limit(action, 'ban', 'unban')
			fh = [x for x in request.POST('hashes', '').split() if len(x) == 32 and util.is_hex(x)]
			if action == 'ban':
				GeneralDataBase().ban_files(fh)
			elif action == 'unban':
				GeneralDataBase().unban_files(fh)
	
		data = GeneralDataBase().get_mod_area_files_data()
		data['section'] = section
		data['msg'] = msg
		return render_to_response(request, 'mod_files.html', data)


def mod_logout(request):
	if sessions.get_mod_logout_confirm(request) == request.GET('c'):
		sessions.delete_mod_login(request)
		return RedirectResponse(request['HTTP_REFERER'])
	else:
		raise Forbidden


def options(request, board, thread_id, post_id):
	thread_id = int(thread_id)
	post_id = int(post_id)
	
	action = request.POST('action')
	
	if request.method == 'GET' or not action:
		data = board.db().get_post_options_data(thread_id, post_id, request['REMOTE_ADDR'], cookies.check_post_verifier(request, board, thread_id, post_id))
		if data is None or data['status'] != 'undeleted':
			raise NotFound
		return render_to_response(request, 'options.html', data)
	
	if request.method == 'POST' and request.local_referer():
		action = limit(action, 'flag', 'cookie_delete', 'tripcode_delete')
		db = board.db()
	
		if not db.post_not_deleted(thread_id, post_id):
			raise NotFound
		
		if action == 'flag':
			board.db().flag_post(thread_id, post_id, request['REMOTE_ADDR'])
			msg = 'Post Flagged.'
		
		elif action == 'cookie_delete':
			if cookies.check_post_verifier(request, board, thread_id, post_id):
				db.delete_post(thread_id, post_id)
				msg = 'Post Deleted.'
			else:
				msg = 'Invalid Cookie.'
		
		elif action == 'tripcode_delete':
			trip = db.get_post_tripcode(thread_id, post_id)
			if tripcodes.check(board, request.POST('tripcode'), trip):
				db.delete_post(thread_id, post_id)
				msg = 'Post Deleted.'
			else:
				msg = 'Invalid Tripcode.'
	
		data = board.db().get_post_options_data(thread_id, post_id, request['REMOTE_ADDR'], cookies.check_post_verifier(request, board, thread_id, post_id), msg)
		if data is None:
			raise NotFound
		return render_to_response(request, 'options.html', data)
	
	raise Forbidden


def mod_options(request, board, thread_id, post_id):
	thread_id = int(thread_id)
	post_id = int(post_id)

	if not get_mod_name(request):
		raise Forbidden
	
	if request.method == 'GET':
		data = board.db().get_post_mod_options_data(thread_id, post_id, GeneralDataBase())
		if data is None:
			raise NotFound
		return render_to_response(request, 'mod_options.html', data)
	
	elif request.method == 'POST' and request.local_referer():
	
		action = limit(request.POST('action'), 'move_thread', 'delete_thread', 'modify_thread', 'modify_post')
		msg = None
		
		if action == 'delete_thread':
			board.db().delete_thread(thread_id)
			return RedirectResponse('/%s/' % board.DIR)
		
		elif action == 'move_thread':
			dest_board = common.get_board(request.POST('dest_board'))
			if dest_board:
				dest_tid = database.move_thread(board, thread_id, dest_board, request['SERVER_NAME'])
				msg = 'moved thread to <a href="/%s/%s">/%s/%s</a>' % ((dest_board.DIR, dest_tid)*2)
		
		elif action == 'modify_thread':
			sticky = 'sticky_thread' in request.post_vars
			lock = 'lock_thread' in request.post_vars
			subject = clamp_str(request.POST('subject'), board.MAX_SUBJECT_LENGTH)
			board.db().edit_thread(thread_id, subject, sticky, lock)
			msg = 'modified thread'
		
		elif action == 'modify_post':
			reviewed = 'reviewed' in request.post_vars
			status = limit(request.POST('status'), 'undeleted', 'user_deleted', 'mod_deleted')
			name = clamp_str(request.POST('name'), board.MAX_NAME_LENGTH)
			tripcode = clamp_str(request.POST('tripcode'), board.TRIPCODE_LENGTH[2])
			comment = clamp_str(request.POST('comment'), board.MAX_COMMENT_LENGTH)
			mod = 'mod' in request.post_vars
			delete_file = 'delete_file' in request.post_vars
			ban_file = 'ban_file' in request.post_vars
			
			db = board.db()
			if ban_file:
				file_md5 = db.get_post_file_md5(thread_id, post_id)
				if file_md5:
					GeneralDataBase().add_file_ban(file_md5)
			db.edit_post(thread_id, post_id, status, name, tripcode, mod, comment, delete_file, reviewed)
			msg = 'modified post'
	
		data = board.db().get_post_mod_options_data(thread_id, post_id, GeneralDataBase(), msg)
		if data is None:
			raise NotFound
		return render_to_response(request, 'mod_options.html', data)
	
	raise Forbidden


handler = common.Handler(
	general_views = (
		View(r'^$', index),
		View(r'^feed/boards$', feed_boards, check_age = False),
		View(r'^confirm_overage$', confirm_overage, check_age = False),
		View(r'^mod$', mod_area, {'section': 'overview'}),
		View(r'^mod_(?P<section>overview|bans|files)$', mod_area),
		View(r'^mod_logout$', mod_logout),
	),
	board_views = (
		View(r'^transload$', transload, check_age = False),
		View(r'^$', view_board, {'page': '0'}),
		View(r'^page(?P<page>[0-9]+)$', view_board),
		View(r'^(?P<id>[0-9]+)$', view_thread),
		View(r'^list', list),
		View(r'^new_thread$', new_thread),
		View(r'^(?P<thread_id>[0-9]+)/reply$', new_reply),
		View(r'^(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/options$', options),
		View(r'^(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/mod_options$', mod_options),
	)
)
