""" important stuff """

#IDEA: option for sorting threads differently: interleave sorted by started with sorted by updated
#IDEA: python 3000
#IDEA: make all code more readable/documented
#IDEA: have the index "/" page display threads/posts from all boards OR boards chosen by user - choices saved in cookie
#IDEA: "thanks" (or other text) indicator for posts (with count not shown unless you "thank" too?)

from __future__ import with_statement


VERSION = '0.4.3'

DEFAULT_HTTP_PORT = 63331

CACHING = True

UPLOADED_FILE_DIR = 'uploaded_files'				#where uploaded files are saved to
STATIC_DIR = 'static'								#where static files (css, js, etc) are located
TEMPLATE_DIR = 'templates'							#where templates are loaded from
CACHE_DIR = 'cache'									#where cached pages will be saved
DATABASE_DIR = 'databases'							#where databases data will be saved
UPLOADED_FILE_URL = '/uploaded_files/'				#prefixed onto urls to uploaded files
STATIC_URL = '/static/'								#prefixed onto urls to static resources

MAX_COMMENT_WORD_LENGTH = 90
MAX_BLURB_LENGTH = 128

BOARD_DIR_REGEX = '^[a-zA-Z0-9_]+$'
MOD_NAME_REGEX = '^[a-zA-Z0-9_]+$'

IMAGE_FILE_EXTS = ['jpg', 'gif', 'png']
TORRENT_FILE_EXTS = ['torrent']
FLASH_FILE_EXTS = ['swf']

MOD_POST_ACTIONS = ('delete', 'delete-and-ban', 'delete-and-ban-file')
DAYS_TO_KEEP_MODLOG = 60								#days to keep modlog entries for


import sys, os, os.path, random, logging, urllib

import pagecache, cookies, uploaded_files, util, sessions, util.http as http, util.templates as templates, util.json as json, database


def board_link(dir):
	return '<a href="/%s/">/%s/</a>' % (dir, dir)


def thread_link(dir, thread_id):
	return '<a href="/%s/%d">/%s/%d</a>' % ((dir, thread_id)*2)


def post_link(dir, thread_id, post_id):
	return '<a href="/%s/%d#%d">/%s/%d#%d</a>' % ((dir, thread_id, post_id)*2)


def check_age(view):
	""" decorator for view functions to enforce the age check. if used on views with a board arg it checks that board otherswise checks if any boards are nsfw. """
	def new_view(request, global_db, **kwargs):
		boards = [kwargs['board_db']] if ('board_db' in kwargs) else global_db.boards
		
		for board_db in boards:
			if board_db.config.show_age_check_page and not cookies.get_overage(request):
				return render_to_response(request, global_db, 'age_check.html')
		
		return view(request, global_db, **kwargs)
	return new_view


def cache(view):
	""" decorator for view functions that returns a cached page under certain conditions. """
	if not CACHING:
		return view
	def new_view(request, global_db, **kwargs):
		pagecache.clean()
		if sessions.get_mod(request) != None:
			return view(request, global_db, **kwargs)
		else:
			key = urllib.quote(request.path, '')
			
			if request.accepts_gzip():
				page = pagecache.load_gzip(key)
				if page:
					return http.Response(page, headers = {'Content-Encoding': 'gzip'})
			else:
				page = pagecache.load(key)
				if page:
					return http.Response(page)
			
			response = view(request, global_db, **kwargs)
			if response.status == 200:
				pagecache.store(key, response.body)
			return response
	return new_view


def mod_login(view):
	""" decorator for view functions that ensures only mods can access the page. """
	def new_view(request, global_db, **kwargs):
		mod_name = sessions.get_mod(request)
		if mod_name is None or global_db.get_mod(mod_name) is None:
			#may be logging in this request
			name = request.post('name')
			password = request.post('password')
			mod = global_db.get_mod(name, password)
			if mod is not None:
				sessions.set_mod(request, name)
				if request.post('return'):
					response = http.RedirectResponse(util.clamp_to_line(request.post('referer', '/')))
					cookies.set_mod(response)
					return response
			else:
				return render_to_response(request, global_db, 'mod_login.html', {'referer': request['referer'] or ''})
		global_db.mod_active(mod_name)
		response = view(request, global_db, **kwargs)
		cookies.set_mod(response)
		return response
	return new_view


def local_referer(view):
	""" decorator for view functions that raises http.Forbidden if not request.local_referer() """
	def new_view(request, global_db, **kwargs):
		if request.local_referer():
			return view(request, global_db, **kwargs)
		else:
			raise http.Forbidden
	return new_view


def render_to_response(request, global_db, path, locals_ = {}):
	return http.Response(render_to_string(request, global_db, path, locals_))


def render_to_string(request, global_db, path, locals_ = None):
	# add global (every page) template vars
	mod_name = sessions.get_mod(request)
	if mod_name not in global_db.mods:
		mod_name = None
	if locals_ is None:
		locals_ = {}
	locals_.update({
		'request': request,
		'global_db': global_db,
		'mod_name': mod_name,
		'mod_admin': global_db.mods[mod_name][1] if mod_name else False,
		'mod_flags': global_db.get_flags() if mod_name else None,
		'mod_logout_confirm': sessions.get_mod_logout_confirm(request),
		'timer': util.Timer(request.time)
	})
	import common
	globals_ = {'sys': sys, 'database': database, 'common': common, 'uploaded_files': uploaded_files, 'json': json, 'util': util, 'random': random}
	return templates.render(TEMPLATE_DIR, path, locals_, globals_)


def delete_board(dir):
	try:
		os.unlink(os.path.join(DATABASE_DIR, 'boards/%s.sqlite' % dir))
		util.rmdirs(os.path.join(UPLOADED_FILE_DIR, dir))
	except:
		logging.exception('error deleting board')


def available_css():
	return [f for f in os.listdir(STATIC_DIR) if f.endswith('.css')]


def bootstrap():
	""" start img2 """
	
	import sys, logging.handlers

	#log to a file and stdout
	logger = logging.getLogger()
	logger.setLevel(logging.ERROR)
	logger.addHandler(logging.StreamHandler())
	fh = logging.handlers.RotatingFileHandler('log', maxBytes = 5*1024*1024, backupCount = 2)
	fh.setFormatter(logging.Formatter('%(asctime)s %(name)s %(levelname)s %(message)s'))
	logger.addHandler(fh)

	if '-x' in sys.argv:
		database.Global().add_default_mod()
		logging.info('default mod added - %s:%s' % database.Global.DEFAULT_MOD)
		sys.exit(0)
	
	try:
		port = int(sys.argv[sys.argv.index('-p') + 1])
		assert 0 < port < 65536
	except:
		port = DEFAULT_HTTP_PORT
	
	pagecache.purge()
	
	spath = os.path.join(DATABASE_DIR, 'sessions.pickle')
	import views
	try:
		util.sessions.unpickle(spath)
		util.http.HTTPServer(('localhost', port), views.Handler(), 1, 10)
		util.sessions.pickle(spath)
	except:
		logging.exception('error running server')
