""" json encoding module """

import StringIO


_STRING_ESCAPES = {
	'"': r'\"',
	'\\': r'\\',
	'/': r'\/',
	'\b': r'\b',	
	'\f': r'\f',
	'\n': r'\n',
	'\r': r'\r',
	'\t': r'\t'
}


def encode(obj, callback = None):
	"""
		encode the supplied object to a json string.
		object must be a nest of dicts, lists, tuples, strings, ints, floats, longs and bools.
	"""
	s = StringIO.StringIO()
	if callback is not None:
		s.write(callback)
		s.write('(')
		_encode_sub(obj, s)
		s.write(')')
	else:
		_encode_sub(obj, s)
	return s.getvalue()
	

def _encode_sub(obj, output):
	
	if isinstance(obj, basestring):
		if isinstance(obj, str):
			obj = obj.decode('utf-8', 'replace')
		output.write('"')
		for c in obj:
			output.write(_STRING_ESCAPES.get(c, c))
		output.write('"')

	elif isinstance(obj, bool):
		output.write('true' if obj else 'false')
	
	elif isinstance(obj, (int, long, float)):
		output.write(str(obj))

	elif isinstance(obj, dict):
		output.write('{')
		for i, (k, v) in enumerate(obj.items()):
			if i:
				output.write(',')
			if not isinstance(k, basestring):
				raise TypeError('dictionary keys must be strings')
			_encode_sub(k, output)
			output.write(':')
			_encode_sub(v, output)
		output.write('}')

	elif isinstance(obj, (list, tuple)):
		output.write('[')
		for i, e in enumerate(obj):
			if i:
				output.write(',')
			_encode_sub(e, output)
		output.write(']')

	elif obj is None:
		output.write('null')

	else:
		raise TypeError('object can not be converted to json: %s' % obj)


if __name__ == '__main__':
	print encode({
    "glossary": {
        "title": "example glossary",
		"GlossDiv": {
            "title": "S",
			"GlossList": {
                "GlossEntry": {
                    "ID": "SGML",
					"SortAs": "SGML",
					"GlossTerm": "Standard Generalized Markup Language",
					"Acronym": "SGML",
					"Abbrev": "ISO 8879:1986",
					"GlossDef": {
                        "para": "A meta-markup language, used to create markup languages such as DocBook.",
						"GlossSeeAlso": ["GML", "XML"]
                    },
					"GlossSee": "markup"
                }
            }
        }
    }
})
	print
	print encode({"menu": {
  "id": "file",
  "value": "File",
  "popup": {
    "menuitem": [
      {"value": "New", "onclick": "CreateNewDoc()"},
      {"value": "Open", "onclick": "OpenDoc()"},
      {"value": "Close", "onclick": "CloseDoc()"}
    ]
  }
}})
	print 
	print encode({"widget": {
    "debug": "on",
    "window": {
        "title": "Sample Konfabulator Widget",
        "name": "main_window",
        "width": 500,
        "height": 500
    },
    "image": { 
        "src": "Images/Sun.png",
        "name": "sun1",
        "hOffset": 250,
        "vOffset": 250,
        "alignment": "center"
    },
    "text": {
        "data": "Click Here",
        "size": 36,
        "style": "bold",
        "name": "text1",
        "hOffset": 250,
        "vOffset": 100,
        "alignment": "center",
        "onMouseUp": "sun1.opacity = (sun1.opacity / 100) * 90;"
    }
}})
	print
	print encode((1, 2, 3))
	print
	print encode([3, 2, 1])
	print
	print encode([]), encode({}), encode(None), encode(False), encode(True), encode('')
	print
	print [encode(k) for k in _STRING_ESCAPES]
	print
	print encode(0)