""" for importing posts/threads from kusaba/wakaba/etc boards """


import re, time, calendar, os.path, sys


def _parse_date(d):
	# find time
	m = re.search(r'(\d{2}):(\d{2})', d)
	if m:
		hour, min = int(m.group(1)), int(m.group(2))
	else:
		hour, min = 0, 0
	
	# find date
	m0 = re.search(r'(\d{2})/(\d{2})/(\d{4})', d)
	m1 = re.search(r'(\d{2})/(\d{2})/(\d{2})', d)
	if m0:
		day, month, year = int(m0.group(1)), int(m0.group(2)), int(m0.group(3))
	elif m1:
		day, month, year = int(m1.group(3)), int(m1.group(2)), int('20' + m1.group(1))
	else:
		day, month, year = 0, 0, 0
	
	try:
		return calendar.timegm([year, month, day, hour, min, 0, 0, 0])
	except:
		return int(time.time())


def _extract_comments(url, page):
	
	comments = []
	
	for i, m in enumerate(re.finditer(r'name">(?P<name>[^<]*).*?</span>(?P<date>[^<]*).*?(No\.)?(?P<id>\d+)</a>.*?blockquote>(?P<comment>.*?)</blockquote', page, re.DOTALL)):
		g = m.groupdict()
		m = re.search(r'(?<=href=")[^"]*?src/(?P<fname>[^"\'>]+?)(?=")', m.group() if i else page)
		g['img'] = m.group('fname') if m else None
		g['id'] = int(g['id'])
		g['name'] = g['name'].strip()
		g['date'] = _parse_date(g['date'])
		comments += [g]
	
	def ref_sub(m):
		id = int(m.group('id'))
		for i, c in enumerate(comments):
			if c['id'] == id:
				return '>>%d' % i
		return '>>%d' % id
	
	def html_ents(s):
		s = s.replace('&lt;', '<').replace('&gt;', '>').replace('&quot;', '"').replace('&amp;', '&')
		return re.sub(r'&#(\d+);', lambda m: unichr(int(m.group(1))), s)
		
	#second pass needed for correct >>X numbers
	for c in comments:
		cc = c['comment']
		cc = re.sub(r'[\r\n]', '', cc)
		cc = re.sub(r'<br[^>]*>', '\r\n', cc)
		cc = re.compile(r'<p>(.*?)</p>', re.DOTALL).sub(r'\1' + '\r\n\r\n', cc)
		cc = cc.strip()
		cc = re.sub(r'<.*?>', '', cc)
		cc = html_ents(cc)
		cc = re.sub(r'>>(?P<id>\d+)', ref_sub, cc)
		c['comment'] = cc
	
	return comments


def transfer(src_dir, board_dir):
	
	import database, uploaded_files, util.http
	
	res, src = os.path.join(src_dir, 'res'), os.path.join(src_dir, 'src')
	threads = [f for f in os.listdir(res) if re.match(r'^\d+\.', f)]

	global_db = database.Global()
	board_db = global_db.get_board(board_dir)
	
	if board_db is None:
		raise ValueError
	
	for thread in threads:
		
		print 'importing %s:\n\t' % thread,
		comments = _extract_comments(thread, open(os.path.join(res, thread), 'r').read().decode('utf-8', 'replace'))
		
		for i, c in enumerate(comments):
			print '%d' % i,
			
			file_md5, file_meta, temp_file = None, None, None
			if c['img']:
				try:
					hup = util.http.UploadedFile(c['img'], open(os.path.join(src, c['img']), 'rb'))
					file_md5, file_meta, temp_file = uploaded_files.check(hup, board_db)
				except:
					print '-unable to read image: "%s"-' % c['img'],
			
	#		if 1:
	#			pprint.pprint(c)
	#			pprint.pprint(file_meta)
	#			continue
			
			if i == 0:
				dt, thread_id = board_db.add_new_thread('', False, False, True, None, None, c['date'], c['name'], '', False, c['comment'], file_md5, file_meta, None, ignore_constraints = True)
			else:
				board_db.add_new_reply(thread_id, c['date'], c['name'], '', False, c['comment'], file_md5, file_meta, None, False, None, None, None, ignore_constraints = True)
			
			if file_md5:
				uploaded_files.save(temp_file, board_db, thread_id, i, file_meta)
			
			print '',
			sys.stdout.flush()
		print

	print 'done: imported %d threads' % len(threads)


if __name__ == '__main__':
	import util
	print util.format_time(_parse_date('dasdad> 09/05/10(Sun)12:19</span'))
	print util.format_time(_parse_date(' 07/05/2009 09:01 </la'))
